pub struct LocalScopeSpawner<'sc, T> { /* private fields */ }Expand description
A spawner that can be obtained from LocalScope::spawner.
This spawner may live longer then the scope.
In case a future is spawned after the scope has been canceled or dropped,
the spawner will return SpawnError::shutdown.
Implementations
Sourceimpl<'sc, T> LocalScopeSpawner<'sc, T>
impl<'sc, T> LocalScopeSpawner<'sc, T>
Sourcepub fn spawn_scoped_local_obj(
&self,
future: LocalFutureObj<'sc, T>,
) -> Result<(), SpawnError>
pub fn spawn_scoped_local_obj( &self, future: LocalFutureObj<'sc, T>, ) -> Result<(), SpawnError>
Spawns a task that polls the given local future.
Errors
This method returns a Result that contains a SpawnError if spawning fails.
Sourcepub fn spawn_local_scoped<Fut>(&self, future: Fut) -> Result<(), SpawnError>where
Fut: Future<Output = T> + 'sc,
pub fn spawn_local_scoped<Fut>(&self, future: Fut) -> Result<(), SpawnError>where
Fut: Future<Output = T> + 'sc,
Spawns a task that polls the given local future.
Errors
This method returns a Result that contains a SpawnError if spawning fails.
Sourceimpl<'sc> LocalScopeSpawner<'sc, ()>
impl<'sc> LocalScopeSpawner<'sc, ()>
Sourcepub fn spawn_local_scoped_with_handle<Fut>(
&self,
future: Fut,
) -> Result<RemoteHandle<Fut::Output>, SpawnError>where
Fut: Future + 'sc,
pub fn spawn_local_scoped_with_handle<Fut>(
&self,
future: Fut,
) -> Result<RemoteHandle<Fut::Output>, SpawnError>where
Fut: Future + 'sc,
Spawns a task that polls the given local future and returns a handle to its output.
Errors
This method returns a Result that contains a SpawnError if spawning fails.